import sys
import ctypes

import cairo
import pytest


def test_type() -> None:
    assert hasattr(cairo, "Glyph")
    assert issubclass(cairo.Glyph, tuple)

    with pytest.raises(TypeError):
        cairo.Glyph()  # type: ignore

    g = cairo.Glyph(0, 0.5, 0.25)
    assert hash(g) == hash(cairo.Glyph(0, 0.5, 0.25))
    assert isinstance(g, tuple)
    assert g == (0, 0.5, 0.25)
    assert g == cairo.Glyph(0, 0.5, 0.25)
    assert g[1] == 0.5
    assert g.index == 0
    assert g.x == 0.5
    assert g.y == 0.25

    with pytest.raises(AttributeError):
        assert g.z  # type: ignore

    assert repr(cairo.Glyph(0, 0, 0)) == \
        "cairo.Glyph(index=0, x=0.0, y=0.0)"
    assert str(cairo.Glyph(0, 0, 0)) == \
        "cairo.Glyph(index=0, x=0.0, y=0.0)"
    assert eval(repr(g)) == g

    assert cairo.Glyph.index
    assert cairo.Glyph.x
    assert cairo.Glyph.y


def test_context() -> None:
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
    context = cairo.Context(surface)

    # We allow tuple[int, float, float] too, but don't expose
    # in the the annotations
    assert context.glyph_extents([(0, 0, 0)])  # type: ignore
    context.glyph_path([(0, 0, 0)])  # type: ignore
    context.show_glyphs([(0, 0, 0)])  # type: ignore

    g = cairo.Glyph(0, 0.5, 0.25)
    assert context.glyph_extents([g])
    context.glyph_path([g])
    context.show_glyphs([cairo.Glyph(0, 0, 0)])

    with pytest.warns(DeprecationWarning, match="num_glyphs.*glyph_extents"):
        context.glyph_extents([], 0)  # type: ignore

    with pytest.warns(DeprecationWarning, match="num_glyphs.*glyph_path"):
        context.glyph_path([], 0)  # type: ignore

    with pytest.raises(TypeError):
        context.glyph_path([object()])  # type: ignore


def test_glyph_limits() -> None:
    max_ulong = 2 ** (ctypes.sizeof(ctypes.c_ulong()) * 8) - 1

    g = cairo.Glyph(max_ulong, sys.float_info.max, -sys.float_info.max)
    assert g.index == max_ulong
    assert g.x == sys.float_info.max
    assert g.y == -sys.float_info.max

    with pytest.raises(OverflowError):
        cairo.Glyph(-1, 0, 0)

    with pytest.raises(OverflowError):
        g = cairo.Glyph(max_ulong + 1, 0, 0)



import ctypes

import cairo
import pytest


def test_region() -> None:
    with pytest.raises(TypeError):
        cairo.Region(object())  # type: ignore

    with pytest.raises(TypeError):
        cairo.Region(object(), object())  # type: ignore

    with pytest.raises(TypeError):
        cairo.Region([object()])  # type: ignore


def test_get_rectangle() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    with pytest.raises(ValueError):
        r.get_rectangle(-1)
    with pytest.raises(ValueError):
        r.get_rectangle(1)
    assert r.get_rectangle(0) == rect
    with pytest.raises(TypeError):
        r.get_rectangle(object())  # type: ignore


def test_rectangles_limits() -> None:
    max_int = 2 ** (ctypes.sizeof(ctypes.c_int()) * 8 - 1) - 1
    min_int = -max_int - 1

    rect = cairo.RectangleInt(max_int, min_int, max_int, min_int)
    assert rect.x == max_int
    assert rect.y == min_int
    assert rect.width == max_int
    assert rect.height == min_int

    with pytest.raises(OverflowError):
        cairo.RectangleInt(max_int + 1, min_int, max_int, min_int)
    with pytest.raises(OverflowError):
        cairo.RectangleInt(min_int, min_int, max_int, min_int - 1)


def test_contains_point() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    assert r.contains_point(0, 0)
    assert not r.contains_point(0, 20)
    with pytest.raises(TypeError):
        r.contains_point(0, object())  # type: ignore


def test_intersect() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    r.intersect(r)
    r.intersect(rect)
    with pytest.raises(TypeError):
        r.intersect(object())  # type: ignore
    with pytest.raises(TypeError):
        r.intersect()  # type: ignore

    assert r.__eq__(object()) == NotImplemented
    assert rect.__eq__(object()) == NotImplemented


def test_equal() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    assert r.equal(r)
    with pytest.raises(TypeError):
        r.equal(object())  # type: ignore
    with pytest.raises(TypeError):
        r.equal()  # type: ignore


def test_subtract() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    r.subtract(r)
    with pytest.raises(TypeError):
        r.subtract(object())  # type: ignore
    with pytest.raises(TypeError):
        r.subtract()  # type: ignore


def test_union() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    r.union(r)
    r.union(rect)
    with pytest.raises(TypeError):
        r.union(object())  # type: ignore
    with pytest.raises(TypeError):
        r.union()  # type: ignore


def test_xor() -> None:
    rect = cairo.RectangleInt(0, 0, 10, 10)
    r = cairo.Region(rect)
    r.xor(r)
    r.xor(rect)
    with pytest.raises(TypeError):
        r.xor(object())  # type: ignore
    with pytest.raises(TypeError):
        r.xor()  # type: ignore


def test_translate() -> None:
    r = cairo.Region()
    r.translate(1, 1)
    with pytest.raises(TypeError):
        r.translate(1, object())  # type: ignore


def test_region_contains_rectangle() -> None:
    rect = cairo.RectangleInt(1, 2, 10, 13)
    region = cairo.Region()
    assert region.contains_rectangle(rect) == cairo.RegionOverlap.OUT
    assert isinstance(region.contains_rectangle(rect), cairo.RegionOverlap)
    with pytest.raises(TypeError):
        region.contains_rectangle(object())  # type: ignore


def test_region_cmp_hash() -> None:
    region = cairo.Region()
    other_region = cairo.Region()
    differ = cairo.Region(cairo.RectangleInt(0, 0, 10, 10))
    with pytest.raises(TypeError):
        hash(region)
    assert region == region
    assert region == other_region
    assert not region != other_region
    assert region != differ

    with pytest.raises(TypeError):
        region < region  # type: ignore

    with pytest.raises(TypeError):
        region > region  # type: ignore

    rect = cairo.RectangleInt(1, 2, 10, 13)
    same = cairo.RectangleInt(1, 2, 10, 13)
    other_rect = cairo.RectangleInt(2, 2, 10, 13)
    with pytest.raises(TypeError):
        hash(rect)

    assert rect == same
    assert rect != other_rect

    with pytest.raises(TypeError):
        rect < same  # type: ignore

    with pytest.raises(TypeError):
        rect > same  # type: ignore



import ctypes

import cairo
import pytest


def test_type() -> None:
    assert hasattr(cairo, "TextCluster")
    assert issubclass(cairo.TextCluster, tuple)

    with pytest.raises(TypeError):
        cairo.TextCluster()  # type: ignore

    r = cairo.TextCluster(2, 1)
    assert hash(r) == hash(cairo.TextCluster(2, 1))
    assert isinstance(r, tuple)
    assert r == (2, 1)
    assert r == cairo.TextCluster(2, 1)
    assert r[1] == 1
    assert r.num_bytes == 2
    assert r.num_glyphs == 1

    with pytest.raises(AttributeError):
        assert r.z  # type: ignore

    assert repr(r) == "cairo.TextCluster(num_bytes=2, num_glyphs=1)"
    assert str(r) == "cairo.TextCluster(num_bytes=2, num_glyphs=1)"
    assert eval(repr(r)) == r

    assert cairo.TextCluster.num_bytes
    assert cairo.TextCluster.num_glyphs


def test_text_cluster_limits() -> None:
    max_int = 2 ** (ctypes.sizeof(ctypes.c_int()) * 8 - 1) - 1
    min_int = -max_int - 1

    tc = cairo.TextCluster(max_int, min_int)
    assert tc.num_bytes == max_int
    assert tc.num_glyphs == min_int

    with pytest.raises(OverflowError):
        cairo.TextCluster(max_int + 1, min_int)
    with pytest.raises(OverflowError):
        cairo.RectangleInt(min_int, min_int - 1)



import sys
import os
import tempfile
import shutil
import platform

import pytest
import cairo


@pytest.fixture(scope='module')
def tempdir_path():
    dir_ = tempfile.mkdtemp()
    try:
        yield dir_
    finally:
        shutil.rmtree(dir_)


class _PathLike:

    def __init__(self, value):
        self._value = value

    def __fspath__(self):
        return self._value


def test_fspaths(tempdir_path) -> None:
    def write(path):
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
        surface.write_to_png(path)

    def get_temp_path(filename):
        filename = os.fspath(filename)
        if isinstance(filename, bytes):
            return os.path.join(os.fsencode(tempdir_path), filename)
        else:
            return os.path.join(tempdir_path, filename)

    def to_test_paths(filenames):
        for f in filenames:
            yield get_temp_path(f)
            yield _PathLike(get_temp_path(f))

    def can_encode(filename: str) -> bool:
        try:
            filename.encode(sys.getfilesystemencoding())
        except UnicodeEncodeError:
            return False
        return True

    FAIL = [
        "foo\x00bar",
        b"foo\x00bar",
    ]
    OK = [
        "foo",
        b"foo",
    ]
    CAIROFAIL = []

    if sys.platform == "linux":
        OK += [
            b"\xFF\xFF\xFF",
            os.fsdecode(b"\xFF\xFF\xFF"),
        ]

    for text in ["\u2764\uFE0F", "öäüß"]:
        if can_encode(text):
            OK += [
                text,
                os.fsencode(text),
            ]

    # https://github.com/pypy/pypy/issues/5259
    if os.name == "nt" and sys.getfilesystemencoding() == "utf-8" and \
            platform.python_implementation() != "PyPy":
        CAIROFAIL += [
            "\uD800",
            os.fsencode("\uD800"),
        ]

    # Supported by Python, but not by cairo
    for p in to_test_paths(CAIROFAIL):
        assert not os.path.exists(p)
        with pytest.raises(ValueError):
            write(p)
        assert not os.path.exists(p)
        with open(p, "wb"):
            pass
        assert os.path.exists(p)
        os.unlink(p)

    # Not supported by Python, and not by cairo
    for p in to_test_paths(FAIL):
        assert not os.path.exists(p)
        with pytest.raises(ValueError):
            write(p)
        # TypeError to work around pypy bug:
        # https://github.com/pypy/pypy/commit/0ecdacc5bde4c6cd79e2
        with pytest.raises((ValueError, TypeError)):
            with open(p, "wb"):
                pass
        assert not os.path.exists(p)

    # Supported by Python, and by cairo
    for p in to_test_paths(OK):
        assert not os.path.exists(p)
        write(p)
        assert os.path.exists(p)
        os.unlink(p)



import cairo
import pytest

# 2.1.3+ required for "BGRA" format
pygame = pytest.importorskip("pygame", minversion="2.1.3")


def test_image_surface_to_pygame_image() -> None:
    width, height = 100, 100
    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
    buf = surface.get_data()
    image = pygame.image.frombuffer(buf, (width, height), "BGRA")
    assert image



import cairo
import pytest


@pytest.fixture
def font_options() -> cairo.FontOptions:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    return surface.get_font_options()


@pytest.fixture
def font_face() -> cairo.FontFace:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    context = cairo.Context(surface)
    return context.get_font_face()


@pytest.fixture
def scaled_font(font_face: cairo.FontFace, font_options: cairo.FontOptions) -> cairo.ScaledFont:
    return cairo.ScaledFont(
        font_face, cairo.Matrix(), cairo.Matrix(), font_options)


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "set_custom_palette_color"),
                    reason="too old cairo")
def test_font_options_custom_palette_color(font_options: cairo.FontOptions) -> None:
    font_options.set_custom_palette_color(42, 0.25, 0.5, 0.75, 1.0)
    with pytest.raises(cairo.Error) as exc_info:
        font_options.get_custom_palette_color(24)
    assert exc_info.value.status == cairo.Status.INVALID_INDEX
    assert font_options.get_custom_palette_color(42) == (0.25, 0.5, 0.75, 1.0)
    assert isinstance(font_options.get_custom_palette_color(42), tuple)


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "set_color_mode"),
                    reason="too old cairo")
def test_font_options_set_color_mode(font_options: cairo.FontOptions) -> None:
    font_options.set_color_mode(cairo.ColorMode.COLOR)
    assert font_options.get_color_mode() == cairo.ColorMode.COLOR
    with pytest.raises(TypeError):
        font_options.set_color_mode(object())  # type: ignore


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "get_color_mode"),
                    reason="too old cairo")
def test_font_options_get_color_mode(font_options: cairo.FontOptions) -> None:
    assert font_options.get_color_mode() == cairo.ColorMode.DEFAULT
    assert isinstance(font_options.get_color_mode(), cairo.ColorMode)


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "set_color_palette"),
                    reason="too old cairo")
def test_font_options_set_color_palette(font_options: cairo.FontOptions) -> None:
    font_options.set_color_palette(42)
    assert font_options.get_color_palette() == 42


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "get_color_palette"),
                    reason="too old cairo")
def test_font_options_get_color_palette(font_options: cairo.FontOptions) -> None:
    assert font_options.get_color_palette() == cairo.COLOR_PALETTE_DEFAULT


@pytest.mark.skipif(not hasattr(cairo.FontOptions, "get_variations"),
                    reason="too old cairo")
def test_font_options_variations(font_options: cairo.FontOptions) -> None:
    assert font_options.get_variations() is None
    font_options.set_variations("foo")
    assert font_options.get_variations() == "foo"
    font_options.set_variations(None)
    assert font_options.get_variations() is None

    with pytest.raises(TypeError):
        font_options.set_variations(1)  # type: ignore

    with pytest.raises(TypeError):
        font_options.set_variations("foo", "bar")  # type: ignore

    with pytest.raises(TypeError):
        font_options.set_variations()  # type: ignore


def test_font_options() -> None:
    assert isinstance(cairo.FontOptions(), cairo.FontOptions)
    with pytest.raises(TypeError):
        cairo.FontOptions(object())  # type: ignore


def test_font_options_copy_equal() -> None:
    surface = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
    font_options = surface.get_font_options()
    font_options.set_hint_metrics(cairo.HINT_METRICS_DEFAULT)
    new = font_options.copy()
    assert font_options.equal(new)
    assert new.get_hint_metrics() == cairo.HINT_METRICS_DEFAULT
    font_options.set_hint_metrics(cairo.HINT_METRICS_ON)
    assert not font_options.equal(new)
    assert new.get_hint_metrics() == cairo.HINT_METRICS_DEFAULT
    with pytest.raises(TypeError):
        font_options.equal(object())  # type: ignore


def test_font_options_hash() -> None:
    surface = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
    font_options = surface.get_font_options()
    assert font_options.hash() == font_options.hash()
    assert isinstance(font_options.hash(), int)


def test_font_options_merge() -> None:
    surface = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
    font_options = surface.get_font_options()
    font_options.set_hint_metrics(cairo.HINT_METRICS_DEFAULT)
    new = font_options.copy()
    new.set_hint_metrics(cairo.HINT_METRICS_ON)
    font_options.merge(new)
    assert font_options.get_hint_metrics() == cairo.HINT_METRICS_ON
    with pytest.raises(TypeError):
        font_options.merge(object())  # type: ignore


def test_font_options_hashable_protocol() -> None:
    # make sure __eq__ and __ne__ work
    surface = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
    font_options = surface.get_font_options()
    assert font_options == font_options.copy()
    assert not font_options != font_options.copy()
    font_options.set_hint_metrics(cairo.HINT_METRICS_DEFAULT)
    different = font_options.copy()
    different.set_hint_metrics(cairo.HINT_METRICS_ON)
    assert font_options != different
    assert not font_options == different
    assert font_options != object()

    # make sure the other operators are undefined
    with pytest.raises(TypeError):
        font_options < font_options  # type: ignore
    assert font_options.__gt__(font_options) is NotImplemented  # type: ignore


def test_font_options_set_antialias(font_options: cairo.FontOptions) -> None:
    font_options.set_antialias(cairo.Antialias.GRAY)
    assert font_options.get_antialias() == cairo.Antialias.GRAY
    with pytest.raises(TypeError):
        font_options.set_antialias(object())  # type: ignore


def test_font_options_set_hint_metrics(font_options: cairo.FontOptions) -> None:
    font_options.set_hint_metrics(cairo.HintMetrics.OFF)
    assert font_options.get_hint_metrics() == cairo.HintMetrics.OFF
    with pytest.raises(TypeError):
        font_options.set_hint_metrics(object())  # type: ignore


def test_font_options_set_hint_style(font_options: cairo.FontOptions) -> None:
    font_options.set_hint_style(cairo.HintStyle.SLIGHT)
    assert font_options.get_hint_style() == cairo.HintStyle.SLIGHT
    with pytest.raises(TypeError):
        font_options.set_hint_style(object())  # type: ignore


def test_font_options_set_subpixel_order(font_options: cairo.FontOptions) -> None:
    font_options.set_subpixel_order(cairo.SubpixelOrder.VRGB)
    assert font_options.get_subpixel_order() == cairo.SubpixelOrder.VRGB
    with pytest.raises(TypeError):
        font_options.set_subpixel_order(object())  # type: ignore


def test_font_face(font_face: cairo.FontFace) -> None:
    with pytest.raises(TypeError):
        cairo.FontFace()  # type: ignore

    assert font_face == font_face
    assert font_face != object()


def test_font_face_cmp_hash() -> None:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    context = cairo.Context(surface)
    ff = context.get_font_face()
    other = context.get_font_face()
    assert ff == other
    assert not ff != other
    assert hash(ff) == hash(other)

    sf = context.get_scaled_font()
    other_sf = context.get_scaled_font()
    assert sf == other_sf
    assert not sf != other_sf
    assert hash(sf) == hash(other_sf)

    fo = context.get_font_options()
    # FontOptions compare by their content and they are mutable, so not
    # hashable.
    with pytest.raises(TypeError):
        hash(fo)  # type: ignore


def test_scaled_font(scaled_font: cairo.ScaledFont) -> None:
    with pytest.raises(TypeError):
        cairo.ScaledFont()  # type: ignore

    assert scaled_font == scaled_font
    assert scaled_font != object()


def test_scaled_font_extents(scaled_font: cairo.ScaledFont) -> None:
    assert isinstance(scaled_font.extents(), tuple)


def test_scaled_font_get_font_face(scaled_font: cairo.ScaledFont) -> None:
    assert isinstance(scaled_font.get_font_face(), cairo.FontFace)


def test_scaled_font_get_scale_matrix(scaled_font: cairo.ScaledFont) -> None:
    assert isinstance(scaled_font.get_scale_matrix(), cairo.Matrix)


def test_scaled_font_text_extents(scaled_font: cairo.ScaledFont) -> None:
    with pytest.raises(TypeError):
        scaled_font.text_extents(object())  # type: ignore


def test_scaled_font_glyph_extents(scaled_font: cairo.ScaledFont) -> None:

    scaled_font.glyph_extents([cairo.Glyph(0, 0.5, 0.25)])

    with pytest.warns(DeprecationWarning, match="num_glyphs.*glyph_extents"):
        scaled_font.glyph_extents([], 0)  # type: ignore

    with pytest.raises(TypeError):
        scaled_font.glyph_extents(object())  # type: ignore
    with pytest.raises(TypeError):
        scaled_font.glyph_extents([object()])  # type: ignore
    with pytest.raises(TypeError):
        scaled_font.glyph_extents()  # type: ignore


def test_toy_font_face() -> None:
    with pytest.raises(TypeError):
        cairo.ToyFontFace(object())  # type: ignore


def test_toy_font_get_family() -> None:
    font_face = cairo.ToyFontFace("")
    assert isinstance(font_face.get_family(), str)
    font_face = cairo.ToyFontFace("serif")
    assert isinstance(font_face.get_family(), str)
    font_face = cairo.ToyFontFace("sans-serif")
    assert isinstance(font_face.get_family(), str)


def test_toy_font_get_slant() -> None:
    font_face = cairo.ToyFontFace("")
    assert font_face.get_slant() == cairo.FontSlant.NORMAL
    assert isinstance(font_face.get_slant(), cairo.FontSlant)


def test_toy_font_get_weight() -> None:
    font_face = cairo.ToyFontFace("")
    assert font_face.get_weight() == cairo.FontWeight.NORMAL
    assert isinstance(font_face.get_weight(), cairo.FontWeight)


def test_font_options_get_antialias(font_options: cairo.FontOptions) -> None:
    assert font_options.get_antialias() == cairo.Antialias.DEFAULT
    assert isinstance(font_options.get_antialias(), cairo.Antialias)


def test_font_options_get_hint_metrics(font_options: cairo.FontOptions) -> None:
    assert font_options.get_hint_metrics() == cairo.HintMetrics.ON
    assert isinstance(font_options.get_hint_metrics(), cairo.HintMetrics)


def test_font_options_get_hint_style(font_options: cairo.FontOptions) -> None:
    assert font_options.get_hint_style() == cairo.HintStyle.DEFAULT
    assert isinstance(font_options.get_hint_style(), cairo.HintStyle)


def test_font_options_get_subpixel_order(font_options: cairo.FontOptions) -> None:
    assert font_options.get_subpixel_order() == cairo.SubpixelOrder.DEFAULT
    assert isinstance(font_options.get_subpixel_order(), cairo.SubpixelOrder)


def test_scaled_font_get_ctm() -> None:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    ctx = cairo.Context(surface)
    sf = ctx.get_scaled_font()
    matrix = sf.get_ctm()
    assert isinstance(matrix, cairo.Matrix)


def test_scaled_font_get_font_matrix() -> None:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    ctx = cairo.Context(surface)
    sf = ctx.get_scaled_font()
    matrix = sf.get_font_matrix()
    assert isinstance(matrix, cairo.Matrix)


def test_scaled_font_get_font_options() -> None:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    ctx = cairo.Context(surface)
    sf = ctx.get_scaled_font()
    font_options = sf.get_font_options()
    assert isinstance(font_options, cairo.FontOptions)


def test_scaled_font_text_to_glyphs() -> None:
    surface = cairo.ImageSurface(cairo.Format(0), 10, 10)
    ctx = cairo.Context(surface)
    sf = ctx.get_scaled_font()
    assert sf.text_to_glyphs(0, 0, "") == ([], [], 0)
    glyphs, clusters, flags = sf.text_to_glyphs(0, 0, "a")
    assert sf.text_to_glyphs(0, 0, "a", True) == (glyphs, clusters, flags)
    assert len(glyphs) == 1
    assert isinstance(glyphs[0], cairo.Glyph)
    assert len(clusters) == 1
    assert isinstance(clusters[0], cairo.TextCluster)
    assert flags == 0
    assert sf.text_to_glyphs(0, 0, "a", False) == glyphs
    glyphs, clusters, flags = sf.text_to_glyphs(0, 0, "a b")
    assert len(glyphs) == 3
    assert glyphs[0] != glyphs[1]
    assert len(clusters) == 3

    with pytest.raises(TypeError):
        sf.text_to_glyphs(object())  # type: ignore



import cairo
import pytest

try:
    from . import cmod
except ImportError:
    pytest.skip("cmod not built", allow_module_level=True)


def test_create_image_surface() -> None:
    surface = cmod.create_image_surface()
    assert isinstance(surface, cairo.ImageSurface)


def test_create_context() -> None:
    context = cmod.create_context()
    assert isinstance(context, cairo.Context)
    assert isinstance(context.get_target(), cairo.ImageSurface)


def test_check_status() -> None:
    cmod.check_status(cairo.STATUS_SUCCESS)

    with pytest.raises(cairo.Error) as excinfo:
        cmod.check_status(cairo.STATUS_NO_MEMORY)
    assert excinfo.value.status == cairo.STATUS_NO_MEMORY

    with pytest.raises(cairo.Error) as excinfo:
        cmod.check_status(9999)
    assert excinfo.value.status == 9999


def test_capi() -> None:
    cmod.test_capi()



import cairo


def create_image_surface() -> cairo.ImageSurface:
    ...


def create_context() -> cairo.Context:
    ...


def check_status(status: int) -> None:
    ...


def test_capi() -> None:
    ...
